SystemOrganization addCategory: #'XML-Tests-Parser'!
SystemOrganization addCategory: #'XML-Tests-Parser-Nodes'!

TestCase subclass: #XMLNodeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Nodes'!

!XMLNodeTest commentStamp: '<historical>' prior: 0!
This is a base class for node tests.!

!XMLNodeTest classSide methodsFor: 'utility' stamp: 'JAAyer 11/20/2010 14:20'!
reorderTests
	self reorderTestsIn: self! !

!XMLNodeTest classSide methodsFor: 'utility' stamp: 'JAAyer 11/24/2010 08:18'!
reorderTests: aSelectorCollection in: aClass
	"
	This utility method reorders test* messages the receiver (a TestCase subclass) implements.

	Given a collection of selectors in a desired order where each begins with a 'test' or 'test' + integer
	prefix, it will rename each such that they have a new 'test' + integer prefix where the integer
	corresponds to their position in the supplied collection.
	
	If the test messages in the receiver are already in the order you like but are not numbered properly,
	use the form of this message that takes no arguments to renumber them properly.
	"

	| oldSource newSource methodCategory digitsNeeded |

	(aSelectorCollection allSatisfy: [:each | each asString beginsWith: 'test'])
		ifFalse: [self error: 'Will not reorder non-test selector'].
	digitsNeeded := aSelectorCollection size printString size.
	aSelectorCollection doWithIndex: [:oldSelector :i |
		oldSource := (aClass sourceCodeAt: oldSelector) asString readStream.
		newSource := (String new: oldSource size) writeStream.
		methodCategory := aClass whichCategoryIncludesSelector: oldSelector.

		oldSource skip: 'test' size.
		[oldSource peek isDigit]
			whileTrue: [oldSource next].

		newSource
			nextPutAll: 'test';
			nextPutAll: (i printPaddedWith: $0 to: digitsNeeded);
			nextPutAll: oldSource upToEnd.

		aClass
			removeSelector: oldSelector;
			compile: newSource contents classified: methodCategory].! !

!XMLNodeTest classSide methodsFor: 'utility' stamp: 'JAAyer 11/21/2010 05:05'!
reorderTestsIn: aClass
	self
		reorderTests:
			((self selectors select: [:each | each asString beginsWith: 'test'])
				sortBy: [:a : b | (a asString compare: b asString) < 3]) asArray
		in: aClass! !

!XMLNodeTest methodsFor: 'tests' stamp: 'JAAyer 10/30/2010 21:51'!
assertNode: aNode hasName: aQualifiedName namespaceURI: aUri
	| prefix localName |

	prefix := aQualifiedName copyUpTo: $:.
	prefix = aQualifiedName
		ifTrue: [
			prefix := ''.
			localName := aQualifiedName]
		ifFalse: [localName := aQualifiedName copyAfter: $:].
	self
		assert: aNode name = aQualifiedName;
		assert: aNode prefix = prefix;
		assert: aNode localName = localName;
		assert: (aNode isNamed: aQualifiedName);
		assert: (aNode isNamed: localName);
		assert: aNode namespaceURI = aUri;
		assert:
			aNode expandedName =
				(aUri
					ifEmpty: [localName]
					ifNotEmpty: ['{', aUri, '}', localName]);
		assert: (prefix ifEmpty: [aNode hasPrefix not] ifNotEmpty: [aNode hasPrefix])

! !

!XMLNodeTest methodsFor: 'assertions' stamp: 'JAAyer 10/20/2010 18:20'!
assertNode: aNode hasPrefix: aPrefix uri: aUri
	self
		assert: aNode prefix = aPrefix;
		assert: aNode namespaceURI = aUri! !

!XMLNodeTest methodsFor: 'assertions' stamp: 'JAAyer 10/28/2010 14:56'!
assertNode: aNode hasType: aType
	self assertNode: aNode hasTypes: (aType ifNotNil: [Array with: aType] ifNil: [#()])! !

!XMLNodeTest methodsFor: 'assertions' stamp: 'JAAyer 10/24/2010 20:06'!
assertNode: aNode hasTypes: aTypeArray
	self deny:
		((#(isAttribute isCData isComment isContentNode isDocument isElement isPI isStringNode)
			copyWithoutAll: aTypeArray) anySatisfy: [:each | aNode perform: each]).
	self assert: (aTypeArray allSatisfy: [:each | aNode perform: each]).! !

!XMLNodeTest methodsFor: 'accessing' stamp: 'JAAyer 3/8/2010 07:21'!
createNodes: aNumber with: aBlock
	^ ((1 to: aNumber)
		collect: [:i | aBlock valueWithPossibleArgument: i])
			asOrderedCollection! !

!XMLNodeTest methodsFor: 'accessing' stamp: 'JAAyer 10/28/2010 21:21'!
escapedString
	^ 'test&lt;&gt;&amp;&quot;'! !

!XMLNodeTest methodsFor: 'accessing' stamp: 'JAAyer 10/20/2010 18:26'!
intersperse: aNodeCollection with: aPaddingCollection
	| mixedCollection nodeStream paddingStream |

	mixedCollection := XMLOrderedList new.
	nodeStream := aNodeCollection readStream.
	paddingStream := aPaddingCollection readStream.
	[nodeStream atEnd and: [paddingStream atEnd]]
		whileFalse: [
			nodeStream atEnd
				ifFalse: [mixedCollection add: nodeStream next].
			paddingStream atEnd
				ifFalse: [mixedCollection add: paddingStream next]].
	^ mixedCollection.! !

!XMLNodeTest methodsFor: 'accessing' stamp: 'JAAyer 7/28/2010 09:10'!
intersperseWithNodes: aNodeCollection
	^ self
		intersperse: aNodeCollection
		with: (self newNodes: aNodeCollection size)! !

!XMLNodeTest methodsFor: 'accessing' stamp: 'JAAyer 3/1/2010 01:10'!
markupString
	^ 'test<>&"'! !

!XMLNodeTest methodsFor: 'accessing' stamp: 'JAAyer 6/11/2010 11:16'!
newNodes: aNumber
	^ self createNodes: aNumber with: [XMLMockNode new]! !

!XMLNodeTest methodsFor: 'accessing' stamp: 'JAAyer 11/15/2010 05:02'!
newStrings: aNumber
	^ self
		createNodes: aNumber
		with: [:i | XMLString string: 'test', i printString]! !

!XMLNodeTest methodsFor: 'assertions' stamp: 'JAAyer 10/28/2010 21:25'!
should: aBlock enumerate: aCollection
	| enumerated |

	enumerated := OrderedCollection new.
	aBlock value: [:each | enumerated add: each].
	self assert: enumerated = aCollection asOrderedCollection.! !

XMLNodeTest subclass: #XMLNodesWithChildrenTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Nodes'!

XMLNodesWithChildrenTest subclass: #XMLDeclarationNodesTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Nodes'!

!XMLDeclarationNodesTest methodsFor: 'assertions' stamp: 'JAAyer 10/30/2010 20:39'!
assertAttributeDecl: anAttributeDeclaration hasElementName: anElementName attributeName: anAttributeName type: aType defaultDecl: aDefaultDeclaration
	| printString |

	self
		assert: anAttributeDeclaration elementName = anElementName;
		assert: anAttributeDeclaration attributeName = anAttributeName;
		assert: anAttributeDeclaration type = aType;
		assert: anAttributeDeclaration defaultDeclaration = aDefaultDeclaration.

	(printString := String new writeStream)
		nextPutAll: '<!!ATTLIST';
		space;
		nextPutAll: anElementName;
		space;
		nextPutAll: anAttributeName;
		space;
		nextPutAll: aType.
	aDefaultDeclaration ifNotEmpty: [
		printString
			space;
			nextPutAll: aDefaultDeclaration].
	printString nextPut: $>.
		
	self assert: anAttributeDeclaration printString = printString contents.! !

!XMLDeclarationNodesTest methodsFor: 'assertions' stamp: 'JAAyer 10/30/2010 20:21'!
assertElementDecl: anElementDeclaration hasName: aName contentModel: aContentModel
	self
		assert: anElementDeclaration name = aName;
		assert: anElementDeclaration contentModel = aContentModel;
		assert: anElementDeclaration printString = ('<!!ELEMENT ', aName, ' ', aContentModel, '>')! !

!XMLDeclarationNodesTest methodsFor: 'assertions' stamp: 'JAAyer 10/30/2010 20:46'!
assertEntityDecl: anEntityDeclaration hasName: aName replacement: aReplacement publicID: aPublicId systemID: aSystemId ndada: anNdata
	| printString |

	self
		assert: anEntityDeclaration name = aName;
		assert: anEntityDeclaration replacement = aReplacement;
		assert: anEntityDeclaration publicID = aPublicId;
		assert: anEntityDeclaration systemID = aSystemId;
		assert: anEntityDeclaration ndata = anNdata.

	(printString := String new writeStream)
		nextPutAll: '<!!ENTITY ';
		nextPutAll: aName;
		space.
	aReplacement
		ifNotEmpty: [
			printString
				nextPut: $";
				nextPutAll: aReplacement;
				nextPut: $"]
		ifEmpty: [
			self
				printExternalIDFromPublicID: aPublicId
				systemID: aSystemId
				on: printString.
			anNdata ifNotEmpty: [
				printString
					space;
					nextPutAll: 'NDATA';
					space;
					nextPutAll: anNdata]].
	printString nextPut: $>.

	self assert: anEntityDeclaration printString = printString contents.! !

!XMLDeclarationNodesTest methodsFor: 'assertions' stamp: 'JAAyer 10/30/2010 20:51'!
assertNotationDecl: aNotationDeclaration hasName: aName publicID: aPublicId systemID: aSystemId
	| printString |

	self
		assert: aNotationDeclaration name = aName;
		assert: aNotationDeclaration publicID = aPublicId;
		assert: aNotationDeclaration systemID = aSystemId.

	(printString := String new writeStream)
		nextPutAll: '<!!NOTATION';
		space;
		nextPutAll: aName;
		space.
	self
		printExternalIDFromPublicID: aPublicId
		systemID: aSystemId
		on: printString.
	printString nextPut: $>.

	self assert: aNotationDeclaration printString = printString contents.! !

!XMLDeclarationNodesTest methodsFor: 'printing' stamp: 'JAAyer 10/30/2010 20:39'!
printExternalIDFromPublicID: aPublicId systemID: aSystemId on: aStream
	aPublicId
		ifNotEmpty: [
			aStream
				nextPutAll: 'PUBLIC';
				space;
				nextPut: $";
				nextPutAll: aPublicId;
				nextPut: $"]
		ifEmpty: [aSystemId ifNotEmpty: [aStream nextPutAll: 'SYSTEM']].
	aSystemId ifNotEmpty: [
		aStream
			space;
			nextPut: $";
			nextPutAll: aSystemId;
			nextPut: $"]
	! !

!XMLDeclarationNodesTest methodsFor: 'tests' stamp: 'JAAyer 11/6/2010 00:02'!
test1attributeDeclarations
	self
		assertAttributeDecl: XMLAttributeDeclarationNode new
		hasElementName: ''
		attributeName: ''
		type: ''
		defaultDecl: ''.
	self
		assertAttributeDecl:
			(XMLAttributeDeclarationNode
				elementName: 'testElement'
				attributeName: 'testAttribute'
				type: 'CDATA'
				defaultDeclaration: '')
		hasElementName: 'testElement'
		attributeName: 'testAttribute'
		type: 'CDATA'
		defaultDecl: ''.
	self
		assertAttributeDecl:
			(XMLAttributeDeclarationNode
				elementName: 'testElement'
				attributeName: 'testAttribute'
				type: 'CDATA'
				defaultDeclaration: '(one|two|three)')
		hasElementName: 'testElement'
		attributeName: 'testAttribute'
		type: 'CDATA'
		defaultDecl: '(one|two|three)'.! !

!XMLDeclarationNodesTest methodsFor: 'tests' stamp: 'JAAyer 11/6/2010 00:02'!
test2elementDeclarations
	self
		assertElementDecl: XMLElementDeclarationNode new
		hasName: ''
		contentModel: ''.
	self
		assertElementDecl:
			(XMLElementDeclarationNode
				name: 'testElement'
				contentModel: '(one,two)')
		hasName: 'testElement'
		contentModel: '(one,two)'.! !

!XMLDeclarationNodesTest methodsFor: 'tests' stamp: 'JAAyer 11/8/2010 19:16'!
test3entityDeclarations
	self
		assertEntityDecl: XMLEntityDeclarationNode new
		hasName: ''
		replacement: ''
		publicID: ''
		systemID: ''
		ndada: ''.
	self
		assertEntityDecl:
			(XMLEntityDeclarationNode
				name: 'testEntity'
				replacement: 'testReplacement')
		hasName: 'testEntity'
		replacement: 'testReplacement'
		publicID: ''
		systemID: ''
		ndada: ''.
	self
		assertEntityDecl:
			(XMLEntityDeclarationNode
				name: 'testEntity'
				publicID: '-//test//test-id//EN'
				systemID: '')
		hasName: 'testEntity'
		replacement: ''
		publicID: '-//test//test-id//EN'
		systemID: ''
		ndada: ''.
	self
		assertEntityDecl:
			(XMLEntityDeclarationNode
				name: 'testEntity'
				publicID: '-//test//test-id//EN'
				systemID: 'http://test')
		hasName: 'testEntity'
		replacement: ''
		publicID: '-//test//test-id//EN'
		systemID: 'http://test'
		ndada: ''.
	self
		assertEntityDecl:
			(XMLEntityDeclarationNode
				name: 'testEntity'
				publicID: ''
				systemID: 'http://test')
		hasName: 'testEntity'
		replacement: ''
		publicID: ''
		systemID: 'http://test'
		ndada: ''.
	self
		assertEntityDecl:
			(XMLEntityDeclarationNode
				name: 'testEntity'
				publicID: '-//test//test-id//EN'
				systemID: 'http://test'
				ndata: 'testNotation')
		hasName: 'testEntity'
		replacement: ''
		publicID: '-//test//test-id//EN'
		systemID: 'http://test'
		ndada: 'testNotation'.! !

!XMLDeclarationNodesTest methodsFor: 'tests' stamp: 'JAAyer 11/6/2010 00:02'!
test4notationDeclarations
	self
		assertNotationDecl: XMLNotationDeclarationNode new
		hasName: ''
		publicID: ''
		systemID: ''.
	self
		assertNotationDecl:
			(XMLNotationDeclarationNode
				name: 'testNotation'
				publicID: '//test//'
				systemID: '')
		hasName: 'testNotation'
		publicID: '//test//'
		systemID: ''.
	self
		assertNotationDecl:
			(XMLNotationDeclarationNode
				name: 'testNotation'
				publicID: '//test//'
				systemID: 'http://test/')
		hasName: 'testNotation'
		publicID: '//test//'
		systemID: 'http://test/'.
	self
		assertNotationDecl:
			(XMLNotationDeclarationNode
				name: 'testNotation'
				publicID: ''
				systemID: 'http://test/')
		hasName: 'testNotation'
		publicID: ''
		systemID: 'http://test/'.! !

!XMLNodesWithChildrenTest methodsFor: 'assertions' stamp: 'JAAyer 11/8/2010 19:19'!
assertNode: aNode hasChildren: aNodeCollection
	self assert: aNode nodes asArray = aNodeCollection asArray! !

!XMLNodesWithChildrenTest methodsFor: 'assertions' stamp: 'JAAyer 10/28/2010 21:28'!
assertNodeHasNoChildren: aNode
	self
		deny: aNode hasChildren;
		assert: aNode nodes isEmpty;
		assert: aNode firstNode isNil;
		assert: aNode lastNode isNil! !

!XMLNodesWithChildrenTest methodsFor: 'assertions' stamp: 'JAAyer 11/1/2010 03:32'!
should: aBlock addNode: aNode to: aParent
	self should: aBlock addNodes: (Array with: aNode) to: aParent! !

!XMLNodesWithChildrenTest methodsFor: 'assertions' stamp: 'JAAyer 11/1/2010 03:36'!
should: aBlock addNodes: aNodeCollection to: aParent
	| nodesAdded |

	self deny: (aNodeCollection anySatisfy: [:each | aParent includesNode: each]).
	(nodesAdded := aBlock value) isCollection
		ifFalse: [nodesAdded := Array with: nodesAdded].
	self
		assert: nodesAdded asArray = aNodeCollection asArray;
		assert:
			(nodesAdded allSatisfy: [:each |
				(aParent includesNode: each) and: [each parent == aParent]])! !

!XMLNodesWithChildrenTest methodsFor: 'assertions' stamp: 'JAAyer 10/28/2010 21:28'!
should: aBlock removeNode: aNode from: aParent
	self should: aBlock removeNodes: (Array with: aNode) from: aParent! !

!XMLNodesWithChildrenTest methodsFor: 'assertions' stamp: 'JAAyer 10/28/2010 21:32'!
should: aBlock removeNodes: aNodeCollection from: aParent
	| nodesRemoved |

	self assert: (aNodeCollection allSatisfy: [:each | aParent includesNode: each]).
	(nodesRemoved := aBlock value) isCollection
		ifFalse: [nodesRemoved := Array with: nodesRemoved].

	self
		assert: nodesRemoved asArray = aNodeCollection asArray;
		deny: (nodesRemoved anySatisfy: [:each | aParent includesNode: each])! !

!XMLNodesWithChildrenTest methodsFor: 'tests' stamp: 'JAAyer 11/6/2010 00:02'!
test1emptyNodeWithChildren
	self assertNodeHasNoChildren: XMLMockNodeWithChildren new! !

!XMLNodesWithChildrenTest methodsFor: 'tests' stamp: 'JAAyer 11/8/2010 19:19'!
test2addingNodes
	| node children |

	node := XMLMockNodeWithChildren new.
	children := self newNodes: 6.
	self
		should: [node addNode: children second] addNode: children second to: node;
		should: [node addNodeFirst: children first] addNode: children first to: node.
	self
		should: [node addNodes: (children copyFrom: 3 to: 4)]
		addNodes: (children copyFrom: 3 to: 4)
		to: node.
	self
		should: [node addNode: children sixth after: children fourth]
		addNode: children sixth
		to: node.
	self
		should: [node addNode: children fifth before: children sixth]
		addNode: children fifth
		to: node.
	self assertNode: node hasChildren: children.

	self
		should: [node addNode: (XMLMockNode new parent: XMLMockNode new)] raise: Error;
		should: [node addNode: children first] raise: Error.! !

!XMLNodesWithChildrenTest methodsFor: 'tests' stamp: 'JAAyer 11/8/2010 19:20'!
test3removingNodes
	| node children |

	node := XMLMockNodeWithElements new.
	children := self newNodes: 5.
	node addNodes: children.
	self
		should: [node removeNode: children first]
		removeNode: children first
		from: node.
	self
		should: [node removeNodes: (children copyFrom: 2 to: 3)]
		removeNodes: (children copyFrom: 2 to: 3)
		from: node.
	self assert: node removeNodes == node.

	self assertNodeHasNoChildren: node.! !

!XMLNodesWithChildrenTest methodsFor: 'tests' stamp: 'JAAyer 11/6/2010 02:40'!
test4accessingAdjacentNodes
	| node children |

	node := XMLMockNodeWithElements new.
	children := self newNodes: 5.
	node addNodes: children.
	children do: [:each | | next previous |
		next := children after: each ifAbsent: [nil].
		previous := children before: each ifAbsent: [nil].
		
		self
			assert: each nextNode == next;
			assert: each previousNode == previous;
			assert: (node nodeAfter: each) == next;
			assert: (node nodeBefore: each) == previous].

	self
		assert: (node nodeAfter: XMLNode new) isNil;
		assert: (node nodeBefore: XMLNode new) isNil.! !

!XMLNodesWithChildrenTest methodsFor: 'tests' stamp: 'JAAyer 11/6/2010 00:02'!
test5accessingFirstAndLastNodes
	| node children |

	node := XMLMockNodeWithElements new.
	self
		assert: node firstNode isNil;
		assert: node lastNode isNil.

	children := self newNodes: 3.
	node addNode: children first.
	self
		assert: node firstNode == children first;
		assert: node lastNode == children first.

	node addNodes: children allButFirst.
	self
		assert: node firstNode == children first;
		assert: node lastNode == children last.

	node removeNode: children first.
	self
		assert: node firstNode == children second;
		assert: node lastNode == children last.! !

!XMLNodesWithChildrenTest methodsFor: 'tests' stamp: 'JAAyer 11/6/2010 00:02'!
test6replacingNodes
	| node children newChildren |

	node := XMLMockNodeWithChildren new.
	children := self newNodes: 3.
	newChildren := self newNodes: 3.
	node addNodes: children.
	self
		should: [node replaceNode: children first with: newChildren first]
		addNode: newChildren first
		to: node.
	self
		should: [node replaceNode: children second with: newChildren second]
		addNode: newChildren second
		to: node.
	self
		should: [node replaceNode: children third with: newChildren third]
		addNode: newChildren third
		to: node.

	self assert: node nodes asOrderedCollection = newChildren asOrderedCollection.! !

!XMLNodesWithChildrenTest methodsFor: 'tests' stamp: 'JAAyer 11/6/2010 00:02'!
test7childNodeEnumeration
	| node nodes |

	node := XMLMockNodeWithElements new.
	self should: [:block | node nodesDo: block] enumerate: #().

	nodes := self newNodes: 5.
	node addNodes: nodes.
	self should: [:block | node nodesDo: block] enumerate: nodes.! !

XMLNodesWithChildrenTest subclass: #XMLNodesWithElementsTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Nodes'!

XMLNodesWithElementsTest subclass: #XMLElementTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Nodes'!

!XMLElementTest methodsFor: 'assertions' stamp: 'JAAyer 10/29/2010 01:41'!
assertElement: anElement hasAttribute: anAttributeName equalTo: anAttributeValue
	self
		assert: (anElement includesAttribute: anAttributeName);
		assert: (anElement attributeAt: anAttributeName) = anAttributeValue;
		assert: (anElement @ anAttributeName) = anAttributeValue;
		assert: (anElement attributeNodeAt: anAttributeName) notNil;
		assert: (anElement attributeNodeAt: anAttributeName) value = anAttributeValue! !

!XMLElementTest methodsFor: 'assertions' stamp: 'JAAyer 11/1/2010 03:56'!
assertElement: anElement hasAttributes: aDictionary
	self assert:
		((aDictionary isMemberOf: Dictionary)
			ifTrue: [(Dictionary newFrom: anElement attributes) = aDictionary]
			ifFalse: [anElement attributes = aDictionary])! !

!XMLElementTest methodsFor: 'assertions' stamp: 'JAAyer 11/1/2010 03:56'!
assertElement: anElement hasContentString: aContentString withStrings: aStringArray
	| combinedStrings |

	self assert: anElement contentString = aContentString.
	combinedStrings := anElement stringNodes collect: [:each | each string].
	self assert: combinedStrings asArray = aStringArray asArray.! !

!XMLElementTest methodsFor: 'assertions' stamp: 'JAAyer 11/1/2010 03:56'!
assertElement: anElement hasName: aName attributes: aDictionary contentString: aString
	self
		assert: (anElement isNamed: aName);
		assertElement: anElement hasAttributes: aDictionary;
		assert: anElement contentString = aString! !

!XMLElementTest methodsFor: 'assertions' stamp: 'JAAyer 10/30/2010 19:51'!
assertNodeHasNoContentNodes: aNode
	super assertNodeHasNoContentNodes: aNode.
	self assertNodeHasNoStringNodes: aNode.! !

!XMLElementTest methodsFor: 'assertions' stamp: 'JAAyer 10/30/2010 19:50'!
assertNodeHasNoStringNodes: aNode
	self
		deny: aNode hasStringNodes;
		assert: aNode stringNodes isEmpty;
		assert: aNode strings isEmpty
	! !

!XMLElementTest methodsFor: 'tests' stamp: 'JAAyer 11/10/2010 04:58'!
test1emptyElement
	self
		assertNode: XMLElement new hasTypes: #(isElement isContentNode);
		assertNodeHasNoChildren: XMLElement new! !

!XMLElementTest methodsFor: 'tests' stamp: 'JAAyer 11/10/2010 04:58'!
test2elementNaming
	| element |

	element := XMLElement new.
	self
		assertNode: element hasName: '' namespaceURI: '';
		assertNode: (element name: 'testElement') hasName: 'testElement' namespaceURI: ''.
	self
		assertNode: (element name: 'testPrefix:testElement' namespaceURI: 'http://test/')
		hasName: 'testPrefix:testElement'
		namespaceURI: 'http://test/'.! !

!XMLElementTest methodsFor: 'tests' stamp: 'JAAyer 11/10/2010 04:58'!
test3elementContentString
	| element str |

	str := self markupString.
	(element := XMLElement new) contentString: str.
	self
		assert: element hasChildren;
		assert: element hasContentNodes;
		assert: element hasStringNodes;
		assert: element contentString = str.

	"Intersperse between text"
	element addNode: XMLMockNode new.
	self
		assertElement: (element addContent: str)
		hasContentString: str, ' ', str
		withStrings: (Array with: str with: str).

	"addContent: should combine neighboring string nodes"
	self
		assertElement: (element addContent: (XMLString string: str))
		hasContentString: str, ' ', str, str
		withStrings: (Array with: str with: str, str).

	element addElement: ((XMLElement named: 'childElementWithContent') addContent: 'test content').
	self assert: (element contentStringAt: 'childElementWithContent') = 'test content'.! !

!XMLElementTest methodsFor: 'tests' stamp: 'JAAyer 11/20/2010 08:03'!
test4elementAttributes
	| element attrs |

	element := XMLElement new.
	self
		assert: (element attributeAt: 'foo') isEmpty;
		assert: (element attributeAt: 'foo' ifAbsent: ['absent']) = 'absent';
		deny: (element includesAttribute: 'foo').

	self
		assert: (element attributeAt: 'foo' put: 'test') = 'test';
		assert: element hasAttributes;
		assertElement: element hasAttribute: 'foo' equalTo: 'test';
		assert: (element attributeAt: 'bar' ifAbsentPut: ['test2']) = 'test2';
		assertElement: element hasAttribute: 'bar' equalTo: 'test2';
		assert: element attributeNames asArray =  #('foo' 'bar').

	self
		assert: element attributeAssociations asArray =
			(Array with: 'foo'->'test' with: 'bar'->'test2').
	self
		assertElement: element
		hasAttributes:
			(OrderPreservingDictionary
				with: 'foo'->'test'
				with: 'bar'->'test2').

	self
		assert: (element removeAttribute: 'foo') = 'test';
		assert: (element removeAttribute: 'bar') = 'test2';
		deny: element hasAttributes.! !

!XMLElementTest methodsFor: 'tests' stamp: 'JAAyer 11/20/2010 08:03'!
test5elementConstructors
	| noAttributes attributes str element |

	noAttributes := OrderPreservingDictionary new.
	self
		assertElement: (XMLElement named: 'foo')
		hasName: 'foo'
		attributes: noAttributes
		contentString: ''.

	attributes := OrderPreservingDictionary with: 'foo'->'test1' with: 'bar'->'test2'.
	self
		assertElement: (XMLElement name: 'foo' attributes: attributes)
		hasName: 'foo'
		attributes: attributes
		contentString: ''.

	str := self markupString.
	self
		assertElement: (XMLElement named: 'foo')
		hasName: 'foo'
		attributes: noAttributes
		contentString: ''.
	self
		assertElement: (XMLElement name: 'foo' attributes: attributes)
		hasName: 'foo'
		attributes: attributes
		contentString: ''.

	element := XMLElement
		name: 'prefix:foo'
		namespaceURI: 'http://foo'
		attributes: attributes.
	self
		assertElement: element hasName: 'prefix:foo' attributes: attributes contentString: '';
		assertNode: element hasPrefix: 'prefix' uri: 'http://foo'.! !

!XMLElementTest methodsFor: 'tests' stamp: 'JAAyer 11/20/2010 08:03'!
test6elementPrinting
	| element attributes |

	element := (XMLElement named: 'element').
	attributes := OrderPreservingDictionary with: 'b'->'two' with: 'a'->'one'.
	self
		assert: element printString = '<element />';
		assert: (element contentString: 'test') printString = '<element>test</element>';
		assert: (element setAttributes: attributes) printString = '<element b="two" a="one">test</element>'! !

!XMLElementTest methodsFor: 'tests' stamp: 'JAAyer 11/10/2010 04:58'!
test7contentNodeEnumeration
	| node nodes elements stringNodes contentNodes |

	node := XMLElement new.
	elements := self newElements: 3.
	stringNodes := self newStrings: 3.
	contentNodes := self intersperse: elements with: stringNodes.
	node addNodes: (self intersperseWithNodes: contentNodes).

	self
		assert: node contentNodes = contentNodes;
		should: [:block | node contentNodesDo: block] enumerate: contentNodes;
		should: [:block | node stringNodesDo: block] enumerate: stringNodes.! !

!XMLNodesWithElementsTest methodsFor: 'assertions' stamp: 'JAAyer 10/28/2010 21:58'!
assertDocument: aDocument hasDoctype: aDoctype version: aVersion encoding: anEncoding
	self
		assert: aDocument doctypeDeclaration = aDoctype;
		assert: aDocument version = aVersion;
		assert: aDocument encoding = anEncoding! !

!XMLNodesWithElementsTest methodsFor: 'assertions' stamp: 'JAAyer 11/8/2010 19:21'!
assertNode: aNode hasElements: anElementCollection
	self assert: aNode elements asArray = anElementCollection asArray! !

!XMLNodesWithElementsTest methodsFor: 'assertions' stamp: 'JAAyer 10/30/2010 19:48'!
assertNodeHasNoChildren: aNode
	super assertNodeHasNoChildren: aNode.
	self assertNodeHasNoContentNodes: aNode! !

!XMLNodesWithElementsTest methodsFor: 'assertions' stamp: 'JAAyer 10/30/2010 19:51'!
assertNodeHasNoContentNodes: aNode
	self
		deny: aNode hasContentNodes;
		assert: aNode contentNodes isEmpty;
		assert: aNode contentString isEmpty;
		assertNodeHasNoElements: aNode! !

!XMLNodesWithElementsTest methodsFor: 'assertions' stamp: 'JAAyer 10/28/2010 23:38'!
assertNodeHasNoElements: aNode
	self
		deny: aNode hasElements;
		assert: aNode elements isEmpty;
		assert: aNode firstElement isNil;
		assert: aNode lastElement isNil! !

!XMLNodesWithElementsTest methodsFor: 'accessing' stamp: 'JAAyer 11/15/2010 05:03'!
newElements: aNumber
	^ self
		createNodes: aNumber
		with: [:i | XMLElement named: 'testElement', i printString]! !

!XMLNodesWithElementsTest methodsFor: 'accessing' stamp: 'JAAyer 10/28/2010 21:58'!
newElementsNamed: aNameArray
	| names qualifiedName element |

	names := aNameArray asOrderedCollection.
	^ XMLOrderedList newFrom:
		(self
			createNodes: aNameArray size
			with: [
				((qualifiedName := names removeFirst) includes: $:)
					ifTrue: [
						qualifiedName splitQualifiedNameInto: [:prefix :localPart |
							element := XMLElement
								name: qualifiedName
								namespaceURI: 'http://', prefix]]
					ifFalse: [element := XMLElement named: qualifiedName].
				element]).! !

!XMLNodesWithElementsTest methodsFor: 'assertions' stamp: 'JAAyer 11/1/2010 03:54'!
should: aBlock addElement: anElement to: aParent
	self should: aBlock addElements: (Array with: anElement) to: aParent! !

!XMLNodesWithElementsTest methodsFor: 'assertions' stamp: 'JAAyer 11/5/2010 22:47'!
should: aBlock addElements: anElementCollection to: aParent
	| elementsAdded |

	self deny: (anElementCollection anySatisfy: [:each | aParent elements includes: each]).
	self
		should: [elementsAdded := aBlock value]
		addNodes: anElementCollection
		to: aParent.
	elementsAdded isCollection
		ifFalse: [elementsAdded := Array with: elementsAdded].
	self
		assert: elementsAdded asArray = anElementCollection asArray;
		assert: (elementsAdded allSatisfy: [:each | aParent elements includes: each]).! !

!XMLNodesWithElementsTest methodsFor: 'assertions' stamp: 'JAAyer 11/1/2010 03:54'!
should: aBlock removeElement: anElement from: aParent
	self should: aBlock removeElements: (Array with: anElement) from: aParent! !

!XMLNodesWithElementsTest methodsFor: 'assertions' stamp: 'JAAyer 11/5/2010 22:48'!
should: aBlock removeElements: anElementCollection from: aParent
	| elementsRemoved |

	self assert: (anElementCollection allSatisfy: [:each | aParent elements includes: each]).
	self
		should: [elementsRemoved := aBlock value]
		removeNodes: anElementCollection
		from: aParent.
	elementsRemoved isCollection
		ifFalse: [elementsRemoved := Array with: elementsRemoved].
	self
		assert: elementsRemoved asArray = anElementCollection asArray;
		deny: (elementsRemoved anySatisfy: [:each | aParent elements includes: each]).! !

!XMLNodesWithElementsTest methodsFor: 'tests' stamp: 'JAAyer 11/21/2010 05:05'!
test01emptyNodeWithElements
	self assertNodeHasNoChildren: XMLMockNodeWithElements new! !

!XMLNodesWithElementsTest methodsFor: 'tests' stamp: 'JAAyer 11/21/2010 05:05'!
test02addingElements
	| node nodes elements children |

	node := XMLMockNodeWithElements new.
	elements := self newElements: 6.
	nodes := self newNodes: 6.
	children := self intersperse: elements with: nodes.

	self
		should: [node addNodes: (elements copyFrom: 2 to: 3)]
		addElements: (elements copyFrom: 2 to: 3)
		to: node.
	self
		should: [node addNode: nodes second after: elements second]
		addNode: nodes second
		to: node.
	self
		should: [node addNode: nodes third after: elements third]
		addNode: nodes third
		to: node.
	self
		should: [node addNodeFirst: elements first]
		addElement: elements first
		to: node.
	self
		should: [node addNode: nodes first before: elements second]
		addNode: nodes first
		to: node.
	self
		should: [node addElement: elements fourth. elements fourth]
		addElement: elements fourth
		to: node.
	self
		should: [node addNode: nodes fourth after: elements fourth]
		addNode: nodes fourth
		to: node.
	self
		should: [node addNode: elements fifth after: nodes fourth]
		addElement: elements fifth
		to: node.
	self
		should: [node addNodes: (nodes allButFirst: 4)]
		addNodes: (nodes allButFirst: 4)
		to: node.
	self
		should: [node addNode: (elements sixth) before: nodes sixth]
		addElement: elements sixth
		to: node.

	self
		assertNode: node hasChildren: children;
		assertNode: node hasElements: elements.

	self
		should: [node addNode: (XMLElement new parent: XMLMockNode new)] raise: Error;
		should: [node addNode: elements first] raise: Error.! !

!XMLNodesWithElementsTest methodsFor: 'tests' stamp: 'JAAyer 11/21/2010 05:05'!
test03removingElements
	| node nodes elements |

	node := XMLMockNodeWithElements new.
	elements := self newElementsNamed: #('testOne' 'testPrefix:testTwo' 'testTwo' 'testThree').
	node addNodes: (self intersperseWithNodes: elements).
	self
		assert: (node removeNode: elements first) == elements first;
		assert: node elements = elements allButFirst;
		assert: (node elementAt: 'testOne') isNil;
		assert: (node elementsAt: 'testOne') isEmpty;
		assert: (node removeNode: elements second) = elements second;
		assert: node elements = (elements last: 2);
		assert: (node elementAt: 'testTwo' namespaceURI: 'http://testPrefix') isNil;
		assert: (node elementsAt: 'testTwo' namespaceURI: 'http://testPrefix') isEmpty;
		assert: (node elementAt: 'testTwo') == elements third;
		assert: (node elementsAt: 'testTwo') = (XMLOrderedList with: elements third);
		assert: (node removeNode: elements third) == elements third;
		assert: node elements = (elements last: 1);
		assert: (node elementAt: 'bar') isNil;
		assert: (node elementsAt: 'bar') isEmpty;
		assert: node firstElement = elements last;
		assert: (node removeNode: elements last) = elements last;
		deny: node hasElements;
		assert: node firstElement isNil.! !

!XMLNodesWithElementsTest methodsFor: 'tests' stamp: 'JAAyer 11/21/2010 05:05'!
test04accessingAdjacentElements
	| node elements |

	node := XMLMockNodeWithElements new.
	elements := self newElements: 3.
	node addNodes: (self intersperseWithNodes: elements).
	elements do: [:each | | next previous |
		next := elements after: each ifAbsent: [nil].
		previous := elements before: each ifAbsent: [nil].
		
		self
			assert: each nextElement == next;
			assert: each previousElement == previous;
			assert: (node elementAfter: each) == next;
			assert: (node elementBefore: each) == previous].

	self
		assert: (node elementAfter: XMLElement new) isNil;
		assert: (node elementBefore: XMLElement new) isNil.! !

!XMLNodesWithElementsTest methodsFor: 'tests' stamp: 'JAAyer 11/21/2010 05:05'!
test05accessingFirstAndLastElements
	| node elements |

	node := XMLMockNodeWithElements new.
	self
		assert: node firstElement isNil;
		assert: node lastElement isNil.

	elements := self newElements: 3.
	node addNode: elements first.
	self
		assert: node firstElement == elements first;
		assert: node lastElement == elements first.

	node addNodes: elements allButFirst.
	self
		assert: node firstElement == elements first;
		assert: node lastElement == elements last.

	node removeNode: elements first.
	self
		assert: node firstElement == elements second;
		assert: node lastElement == elements last.

	node removeNodes.
	node addNodes: (self intersperseWithNodes: elements).
	self
		assert: node firstElement == elements first;
		assert: node lastElement == elements last.! !

!XMLNodesWithElementsTest methodsFor: 'tests' stamp: 'JAAyer 11/21/2010 05:05'!
test06chlidNodeAndElementEnumeration
	| node nodes elements |

	node := XMLMockNodeWithElements new.
	self
		should: [:block | node nodesDo: block] enumerate: #();
		should: [:block | node elementsDo: block] enumerate: #().

	elements := self newElements: 3.
	nodes := self intersperseWithNodes: elements.
	node addNodes: nodes.
	self
		should: [:block | node nodesDo: block] enumerate: nodes;
		should: [:block | node elementsDo: block] enumerate: elements.! !

!XMLNodesWithElementsTest methodsFor: 'tests' stamp: 'JAAyer 11/21/2010 05:05'!
test07elementAccessing
	| node nodes elements |

	node := XMLMockNodeWithElements new.
	elements := self newElementsNamed: #('testOne' 'testTwo' 'testPrefix:testOne' 'testThree').
	node addNodes: (self intersperseWithNodes: elements).
	self
		assert: (node elementAt: 'absent') isNil;
		assert: (node elementAt: 'absent' ifAbsent: ['absent']) = 'absent';
		assert: (node elementsAt: 'absent') isEmpty;
		assert: (node elementAtAny: #('absent')) isNil;
		assert: (node elementAtAny: #('absent') ifAbsent: ['absent']) = 'absent';
		assert: (node elementAt: 'testOne') == elements first;
		assert: (node elementAt: 'testTwo') == elements second;
		assert: (node elementsAt: 'testOne') = ((elements first: 3) copyWithout: elements second);
		assert: (node elementAt: 'testPrefix:testOne') == elements third;
		assert: (node elementsAt: 'testPrefix:testOne') = (XMLOrderedList with: elements third);
		assert: (node elementAtAny: #('testTwo' 'testPrefix:testOne')) == elements second;
		assert: (node elementAtAny: #('absent' 'testThree')) == elements fourth;
		assert: (node elementAt: 'testOne' namespaceURI: 'absent') isNil;
		assert: (node elementAt: 'absent' namespaceURI: 'http://testPrefix') isNil;
		assert: (node elementAt: 'testOne' namespaceURI: 'absent' ifAbsent: ['absent']) = 'absent';
		assert: (node elementAt: 'absent' namespaceURI: 'http://testPrefix' ifAbsent: ['absent']) = 'absent';
		assert: (node elementAt: 'testOne' namespaceURI: 'http://testPrefix') == elements third;
		assert: (node elementsAt: 'testOne' namespaceURI: 'absent') isEmpty;
		assert: (node elementsAt: 'absent' namespaceURI: 'http://testPrefix') isEmpty;
		assert: (node elementsAt: 'testOne' namespaceURI: 'http://testPrefix') = (elements copyFrom: 3 to: 3)! !

!XMLNodesWithElementsTest methodsFor: 'tests' stamp: 'JAAyer 11/21/2010 05:05'!
test08firstTagSearching
	| node elements |

	node := XMLMockNodeWithElements new.
	elements := self newElementsNamed: #('foo' 'bar' 'prefix:foo' 'prefix:bar' 'bar' 'baz').
	node addNodes: (elements first: 2).
	node nodes first addNodes: (elements copyFrom: 3 to: 4).
	node nodes second addNodes: (elements copyFrom: 5 to: 6).
	self
		assert: (node firstTagNamed: 'absent') isNil;
		assert: (node firstTagNamed: 'foo' with: [false]) isNil;
		assert: (node firstTagNamedAny: #('absent')) isNil;
		assert: (node firstTagNamedAny: #('foo') with: [false]) isNil.

	self
		assert: (node firstTagNamed: 'foo') = elements first;
		assert: (node firstTagNamed: 'prefix:bar') = elements fourth;
		assert: (node firstTagNamedAny: #('prefix:foo' 'prefix:bar' 'absent')) = elements third;
		assert: (node firstTagNamedAny: #('absent:bar' 'baz')) = elements last;
		assert: (elements first firstTagNamed: 'foo') = elements first;
		assert: (elements second firstTagNamedAny: #('foo' 'bar')) = elements second.
	self assert: (node
		firstTagNamed: 'bar'
		with: [:each | each = elements fifth]) = elements fifth.
	self assert: (node
		firstTagNamedAny: #('bar' 'baz')
		with: [:each | each = elements last]) = elements last.
	self assert: (elements first
		firstTagNamed: 'foo'
		with: [:each | each = elements third]) = elements third.
	self assert: (elements second
		firstTagNamedAny: #('bar')
		with: [:each | each = elements fifth]) = elements fifth.! !

!XMLNodesWithElementsTest methodsFor: 'tests' stamp: 'JAAyer 11/21/2010 05:05'!
test09tagsNamedSearching
	| node elements |

	node := XMLMockNodeWithElements new.
	elements := self newElementsNamed: #('foo' 'bar' 'prefix:foo' 'bar' 'prefix:bar' 'baz').
	node addNodes: (elements first: 2).
	node nodes first addNodes: (elements copyFrom: 3 to: 4).
	node nodes second addNodes: (elements copyFrom: 5 to: 6).
	self
		should: [:block | node tagsNamed: 'absent' do: block] enumerate: #();
		should: [:block | node tagsNamedAny: #('absent') do: block] enumerate: #();
		should: [:block | node elementsAt: 'absent' do: block] enumerate: #().

	self
		should: [:block | node tagsNamed: 'foo' do: block]
		enumerate: (Array with: elements first with: elements third).
	self
		should: [:block | node tagsNamed: 'prefix:foo' do: block]
		enumerate: (Array with: elements third).
	self
		should: [:block | node firstElement elementsAt: 'foo' do: block]
		enumerate: (Array with: elements third).
	self
		should: [:block | node tagsNamedAny: #('foo' 'prefix:bar') do: block]
		enumerate: (Array with: elements first with: elements third with: elements fifth)! !

!XMLNodesWithElementsTest methodsFor: 'tests' stamp: 'JAAyer 11/21/2010 05:05'!
test10elementRenaming
	| node nodes elements |

	node := XMLMockNodeWithElements new.
	elements := self newElementsNamed: #('foo' 'prefix:bar' 'bar' 'baz').
	node addNodes: (self intersperseWithNodes: elements).

	elements first name: 'notfoo'.
	self
		deny: (node includesElement: 'foo');
		assert: (node elementAt: 'foo') isNil;
		assert: (node elementsAt: 'foo') isEmpty;
		assert: (node elementAt: 'notfoo') = elements first.

	elements third name: 'prefix:bar' namespaceURI: 'http://bar'.
	self assert: (node elementsAt: 'prefix:bar') = (elements copyFrom: 2 to: 3).

	elements fourth name: 'bar'.
	self assert: (node elementsAt: 'bar') = elements allButFirst.

	elements fourth name: 'foo'.
	self assert: (node elementAt: 'foo') = elements last.! !

!XMLNodesWithElementsTest methodsFor: 'tests' stamp: 'JAAyer 11/21/2010 05:05'!
test11documentTest
	| doc root doctype |

	doc := XMLDocument new.
	root := XMLElement new.
	doctype := XMLDoctypeDeclaration root: 'root' publicID: 'public' systemID: 'system'.
	self
		assert: doc isDocument;
		assert: doc root isNil;
		assert: (doc addElement: root) root = root;
		assertDocument: doc hasDoctype: nil version: nil encoding: 'UTF-8'.

	self
		assertDocument: (doc doctypeDeclaration: doctype)
		hasDoctype: doctype
		version: nil
		encoding: 'UTF-8'.
	self
		assertDocument: (doc version: 1.0)
		hasDoctype: doctype
		version: 1.0
		encoding: 'UTF-8'.
	self
		assertDocument: (doc encoding: 'ISO Latin-1')
		hasDoctype: doctype
		version: 1.0
		encoding: 'ISO Latin-1'.! !

XMLNodeTest subclass: #XMLNodesWithoutChildrenTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Nodes'!

!XMLNodesWithoutChildrenTest methodsFor: 'assertions' stamp: 'JAAyer 10/30/2010 21:02'!
assertAttribute: anAttribute hasName: aName value: aValue
	self
		assert: anAttribute name = aName;
		assert: (anAttribute isNamed: aName);
		assert: anAttribute value = aValue;
		assert: anAttribute printString = (aName, '="', aValue, '"')! !

!XMLNodesWithoutChildrenTest methodsFor: 'assertions' stamp: 'JAAyer 10/29/2010 16:30'!
assertCData: aCDataNode hasString: aString
	self
		assertStringNode: aCDataNode
		hasString: aString
		printString: '<!![CDATA[', aString, ']]>'! !

!XMLNodesWithoutChildrenTest methodsFor: 'assertions' stamp: 'JAAyer 10/29/2010 06:21'!
assertComment: aComment hasString: aString
	self
		assert: aComment string = aString;
		assert: aComment printString = ('<!!--', aString, '-->')! !

!XMLNodesWithoutChildrenTest methodsFor: 'assertions' stamp: 'JAAyer 10/28/2010 21:29'!
assertNode: anObservedNode canNotifyNodes: anObserverArray
	anObservedNode notifyOf: #mockNotifyWith: with: #one.
	self assert: (anObserverArray allSatisfy: [:each | each lastNotificationArguments = #(one)]).

	anObservedNode notifyOf: #mockNotifyWith:with: with: #one with: #two.
	self assert: (anObserverArray allSatisfy: [:each | each lastNotificationArguments = #(one two)]).

	anObservedNode notifyOf: #mockNotifyWith:with:with: with: #one with: #two with: #three.
	self assert: (anObserverArray allSatisfy: [:each | each lastNotificationArguments = #(one two three)]).

	anObserverArray do: [:each | each lastNotificationArguments: nil].
	anObservedNode notifyOf: #mockNotifyWith:with:with: withArguments: #(one two three).
	self assert: (anObserverArray allSatisfy: [:each | each lastNotificationArguments = #(one two three)]). ! !

!XMLNodesWithoutChildrenTest methodsFor: 'assertions' stamp: 'JAAyer 10/28/2010 21:19'!
assertPI: aNode hasTarget: aTarget data: aData
	self
		assert: aNode target = aTarget;
		assert: aNode data = aData;
		assert: aNode printString = ('<?', aTarget, ' ', aData, '?>')! !

!XMLNodesWithoutChildrenTest methodsFor: 'assertions' stamp: 'JAAyer 10/29/2010 00:30'!
assertStringNode: aStringNode hasString: aString printString: aPrintString
	self
		assert: aStringNode string = aString;
		assert: aStringNode contentString = aString;
		assert: aStringNode printString = aPrintString! !

!XMLNodesWithoutChildrenTest methodsFor: 'tests' stamp: 'JAAyer 11/15/2010 05:02'!
test1basicNodeProtocol
	| node parent |

	node := XMLMockNode new.
	self
		assertNode: node hasType: nil;
		deny: node hasChildren;
		assert: (#(nextNode previousNode parent) allSatisfy: [:each | (node perform: each) isNil]);
		assert: node printString = (node class name asString, '(', node identityHash printString, ') ');
		assert: node contentString isEmpty.

	parent := XMLMockNode new.
	node parent: parent.
	self
		assert: node hasParent;
		assert: node parent == parent.! !

!XMLNodesWithoutChildrenTest methodsFor: 'tests' stamp: 'JAAyer 11/6/2010 00:05'!
test2nodeNotification
	| observedNode firstObserverNode secondObserverNode |

	observedNode := XMLMockNode new.
	self
		shouldnt: [observedNode notifyOf: #invalid: with: #one] raise: Error;
		shouldnt: [observedNode notifyOf: #invalid:invalid: with: #one with: #two] raise: Error;
		shouldnt: [observedNode notifyOf: #invalid:invalid:invalid: with: #one with: #two with: #three] raise: Error;
		shouldnt: [observedNode notifyOf: #invalid:invalid:invalid: withArguments: #(one two three)] raise: Error.

	firstObserverNode := XMLMockObserverNode new.
	observedNode addObserver: firstObserverNode.
	self assertNode: observedNode canNotifyNodes: (Array with: firstObserverNode).

	secondObserverNode := XMLMockObserverNode new.
	observedNode addObserver: secondObserverNode.
	self
		assertNode: observedNode
		canNotifyNodes: (Array with: firstObserverNode with: secondObserverNode).

	observedNode removeObserver: firstObserverNode.
	self assertNode: observedNode canNotifyNodes: (Array with: secondObserverNode).

	self
		should: [observedNode notifyOf: #invalid: with: #one] raise: Error;
		should: [observedNode notifyOf: #invalid:invalid: with: #one with: #two] raise: Error;
		should: [observedNode notifyOf: #invalid:invalid:invalid: with: #one with: #two with: #three] raise: Error;
		should: [observedNode notifyOf: #invalid:invalid:invalid: withArguments: #(one two three)] raise: Error.! !

!XMLNodesWithoutChildrenTest methodsFor: 'tests' stamp: 'JAAyer 11/6/2010 00:03'!
test3processingInstructionNodes
	| pi t d |

	pi := XMLPI new.
	t := 'target'.
	d := 'data'.
	self
		assertNode: pi hasType: #isPI;
		assertPI: pi hasTarget: '' data: '';
		assertPI: (pi target: t) hasTarget: t data: '';
		assertPI: (pi data: d) hasTarget: t data: d;
		assertPI: (XMLPI target: t data: d) hasTarget: t data: d.! !

!XMLNodesWithoutChildrenTest methodsFor: 'tests' stamp: 'JAAyer 11/6/2010 00:03'!
test4stringNodes
	| node raw escaped |

	node := XMLString new.
	raw := self markupString.
	escaped := self escapedString.
	self
		assertNode: node hasTypes: #(isStringNode isContentNode);
		assertStringNode: node hasString: '' printString: '';
		assertStringNode: (node string: raw) hasString: raw printString: escaped;
		assertStringNode: (node addContent: raw) hasString: raw, raw printString: escaped, escaped.
	self
		assertStringNode: (node addContent: (XMLString string: raw))
		hasString: raw, raw, raw
		printString: escaped, escaped, escaped.

	self
		assertStringNode: (XMLString string: raw)
		hasString: raw
		printString: escaped.
	self
		assertStringNode: (XMLString string: (XMLString string: raw))
		hasString: raw
		printString: escaped.

	self
		assertStringNode: raw asXMLString hasString: raw printString: escaped;
		assert: node asXMLString == node.! !

!XMLNodesWithoutChildrenTest methodsFor: 'tests' stamp: 'JAAyer 11/6/2010 00:03'!
test5cdataNode
	| node raw |

	node := XMLCData new.
	raw := self markupString.
	self
		assertNode: node hasTypes: #(isCData isStringNode isContentNode);
		assertCData: node hasString: '';
		assertCData: (node string: raw) hasString: raw;
		assertCData: (node addContent: raw) hasString: raw, raw;
		assertCData: (node addContent: raw asXMLString) hasString: raw, raw, raw;
		assert: node asXMLString == node! !

!XMLNodesWithoutChildrenTest methodsFor: 'tests' stamp: 'JAAyer 11/6/2010 00:03'!
test6commentNodes
	| comment raw escaped |

	comment := XMLComment new.
	self
		assertNode: comment hasType: #isComment;
		assertComment: comment hasString: '';
		assertComment: (comment string: 'test') hasString: 'test';
		assertComment: (XMLComment string: 'test') hasString: 'test'.! !

!XMLNodesWithoutChildrenTest methodsFor: 'tests' stamp: 'JAAyer 11/6/2010 00:03'!
test7attributeNodes
	| attribute element |

	attribute := XMLAttribute new.
	self
		assertNode: attribute hasType: #isAttribute;
		assertAttribute: attribute hasName: '' value: '';
		assertNode: attribute hasName: '' namespaceURI: '';
		assertAttribute: (attribute name: 'name') hasName: 'name' value: '';
		assertNode: attribute hasName: 'name' namespaceURI: '';
		assertAttribute: (attribute value: 'value') hasName: 'name' value: 'value';
		assertAttribute: (XMLAttribute name: 'name' value: 'value') hasName: 'name' value: 'value';
		assertNode: (attribute prefix: 'prefix') hasName: 'prefix:name' namespaceURI: '';
		assertNode: (XMLAttribute name: 'prefix:name' value: 'value') hasName: 'prefix:name' namespaceURI: ''.

	element := XMLElement new.
	self
		assert: attribute element isNil;
		assert: (attribute element: element) element == element.! !

TestCase subclass: #XMLParserElementFactoryTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser'!

!XMLParserElementFactoryTest commentStamp: '<historical>' prior: 0!
This class tests the pluggable element factory. To do this it creates a number of dummy subclasses of XMLElement in its #setUp message that should be removed from the system by its #tearDown message. Since these classes only exist during testing, special accessor messages are used to reference them. These accessors are also generated automatically, whenever you send the class #createDummyElementAccessors. The names for the classes and accessors are taken from the class-side #dummyElementNames message and some supporting messages that specify a prefix and suffix (so the created classes don't collide with other, possibly pre-existing classes in the sytem).

Browse the class side for more information.!

!XMLParserElementFactoryTest classSide methodsFor: 'utility' stamp: 'JAAyer 11/22/2010 15:18'!
createDummyElementAccessors
	"self createDummyElementAccessors"

	| sourceStream |

	self removeDummyElementAccessors.
	self dummyElementNames
		do: [:each |
			(sourceStream := String new writeStream)
				nextPut: each first asLowercase;
				nextPutAll: each allButFirst, self dummyElementClassSuffix, 'Class';
				cr;
				tab;
				nextPutAll: '^ Smalltalk at: #';
				nextPutAll: (self toDummyElementClassName: each) asString.

			self
				compile: sourceStream contents
				classified: self dummyElementAccessorCategory].! !

!XMLParserElementFactoryTest classSide methodsFor: 'running' stamp: 'JAAyer 11/20/2010 13:29'!
createDummyElementClasses
	self dummyElementClassNames do: [:each |
		XMLElement
			subclass: each
			instanceVariableNames: ''
			classVariableNames: ''
			poolDictionaries: ''
			category: self category]! !

!XMLParserElementFactoryTest classSide methodsFor: 'accessing' stamp: 'JAAyer 11/20/2010 14:02'!
dummyElementAccessorCategory
	^ #'dummy element classes'! !

!XMLParserElementFactoryTest classSide methodsFor: 'accessing' stamp: 'JAAyer 11/20/2010 13:25'!
dummyElementClassNames
	^ self dummyElementNames collect: [:each | self toDummyElementClassName: each]! !

!XMLParserElementFactoryTest classSide methodsFor: 'accessing' stamp: 'JAAyer 11/20/2010 12:52'!
dummyElementClassPrefix
	^ 'XMLTEST'! !

!XMLParserElementFactoryTest classSide methodsFor: 'accessing' stamp: 'JAAyer 11/20/2010 12:52'!
dummyElementClassSuffix
	^ 'Element'! !

!XMLParserElementFactoryTest classSide methodsFor: 'accessing' stamp: 'JAAyer 11/20/2010 15:35'!
dummyElementNames
	"Dummy element class names will be formed from the following base names plus a prefix and suffix.
	
	self createDummyElementAccessors."

	^ #('Default' 'User' 'Username' 'RealName' 'Date' 'Table' 'SerializationFormat' 'Wrong')! !

!XMLParserElementFactoryTest classSide methodsFor: 'utility' stamp: 'JAAyer 11/20/2010 14:10'!
removeDummyElementAccessors
	(self methodsInCategory: self dummyElementAccessorCategory)
		do: [:each | self removeSelector: each].! !

!XMLParserElementFactoryTest classSide methodsFor: 'running' stamp: 'JAAyer 11/20/2010 14:10'!
removeDummyElementClasses
	self dummyElementClassNames do: [:each | (Smalltalk at: each) removeFromSystem]! !

!XMLParserElementFactoryTest classSide methodsFor: 'accessing' stamp: 'JAAyer 11/20/2010 14:10'!
toDummyElementClassName: aName
	^ (self dummyElementClassPrefix,
		aName,
		self dummyElementClassSuffix) asSymbol! !

!XMLParserElementFactoryTest methodsFor: 'dummy element classes' stamp: 'JAAyer 11/20/2010 15:45'!
dateElementClass
	^ Smalltalk at: #XMLTESTDateElement! !

!XMLParserElementFactoryTest methodsFor: 'dummy element classes' stamp: 'JAAyer 11/20/2010 15:45'!
defaultElementClass
	^ Smalltalk at: #XMLTESTDefaultElement! !

!XMLParserElementFactoryTest methodsFor: 'dummy element classes' stamp: 'JAAyer 11/20/2010 15:45'!
realNameElementClass
	^ Smalltalk at: #XMLTESTRealNameElement! !

!XMLParserElementFactoryTest methodsFor: 'source' stamp: 'JAAyer 11/20/2010 15:45'!
sampleUserXML
	"
	XMLWriter new
		enablePrettyPrinting;
		writeWith: [:writer |
			writer tag
				name: 'user:user';
				xmlnsAt: 'user' put: 'urn:user';
				with: [
					writer
						tag: 'user:username' with: 'jsmith';
						tag: 'user:real-name' with: 'John Smith';
						tag: 'user:date-registered' with: (DateAndTime now - 1 week) printString;
						tag: 'user:date-last-active' with: DateAndTime now printString;
						tag: 'table-name' with: 'users';
						tag: 'row-id' with: (Random new next * 100) asInteger printString;
						tag: 'serializationFormat' with: 'XML']]
	"

	^ ' <user:user xmlns:user="urn:user">
    <user:username>jsmith</user:username>
    <user:real-name>John Smith</user:real-name>
    <user:date-registered>2010-11-13T14:35:57-05:00</user:date-registered>
    <user:date-last-active>2010-11-20T14:35:57-05:00</user:date-last-active>
    <table-name>users</table-name>
    <row-id>61</row-id>
    <serializationFormat>XML</serializationFormat>
</user:user>'! !

!XMLParserElementFactoryTest methodsFor: 'dummy element classes' stamp: 'JAAyer 11/20/2010 15:45'!
serializationFormatElementClass
	^ Smalltalk at: #XMLTESTSerializationFormatElement! !

!XMLParserElementFactoryTest methodsFor: 'running' stamp: 'JAAyer 11/20/2010 13:29'!
setUp
	self class createDummyElementClasses! !

!XMLParserElementFactoryTest methodsFor: 'dummy element classes' stamp: 'JAAyer 11/20/2010 15:45'!
tableElementClass
	^ Smalltalk at: #XMLTESTTableElement! !

!XMLParserElementFactoryTest methodsFor: 'running' stamp: 'JAAyer 11/20/2010 13:29'!
tearDown
	self class removeDummyElementClasses! !

!XMLParserElementFactoryTest methodsFor: 'tests' stamp: 'JAAyer 11/23/2010 15:47'!
testDefaultElementAccessor
	| document |

	document :=
		(XMLDOMParser on: self sampleUserXML)
			nodeFactory: XMLPluggableElementFactory new;
			parseDocument.
	self assert: (document searchWith: [:each | (each isMemberOf: XMLElement) not]) isNil.

	document :=
		(XMLDOMParser on: self sampleUserXML)
			nodeFactory: (XMLPluggableElementFactory new elementClass: self defaultElementClass);
			parseDocument.
	self assert: (document searchWith: [:each | (each isMemberOf: self defaultElementClass) not]) isNil.! !

!XMLParserElementFactoryTest methodsFor: 'tests' stamp: 'JAAyer 11/21/2010 03:28'!
testElementClassNameMatching
	| document |

	document :=
		(XMLDOMParser on: self sampleUserXML)
			nodeFactory:
				(XMLPluggableElementFactory new
					handleElementsMatchingClass: self userElementClass;
					handleElementsMatchingClasses:
						(Array
							with: self realNameElementClass
							with: self serializationFormatElementClass));
			parseDocument.

	self
		assert: ((document root) isMemberOf: self userElementClass);
		assert: ((document root elementAt: 'real-name') isMemberOf: self realNameElementClass);
		assert:
			((document root elementAt: 'serializationFormat')
				isMemberOf: self serializationFormatElementClass).! !

!XMLParserElementFactoryTest methodsFor: 'tests' stamp: 'JAAyer 11/20/2010 18:08'!
testElementNameAndNamespaceMatching
	| factory document root |

	(factory := XMLPluggableElementFactory new)
		elementClass: self defaultElementClass;
		handleElement: 'user:user' namespaceURI: 'urn:user' withClass: self userElementClass;
		handleElement: 'user:user' withClass: self wrongElementClass;
		handleElement: 'user' namespaceURI: 'urn:user' withClass: self wrongElementClass;
		handleElement: 'user' withClass: self wrongElementClass;
		handleElement: 'user:username' withClass: self usernameElementClass;
		handleElement: 'username' namespaceURI: 'urn:user' withClass: self wrongElementClass;
		handleElement: 'username' withClass: self wrongElementClass;
		handleElement: 'real-name' namespaceURI: 'urn:user' withClass: self realNameElementClass;
		handleElement: 'real-name' withClass: self wrongElementClass;
		handleElement: 'date-registered' withClass: self dateElementClass;
		handleElement: 'user:date-last-active' namespaceURI: 'urn:notuser' withClass: self wrongElementClass;
		handleElement: 'notuser:date-last-active' namespaceURI: 'urn:user' withClass: self wrongElementClass;
		handleElement: 'date-last-active' namespaceURI: 'urn:notuser' withClass: self wrongElementClass;
		handleElement: 'notuser:date-last-active' withClass: self wrongElementClass.

	document :=
		(XMLDOMParser on: self sampleUserXML)
			nodeFactory: factory;
			parseDocument.
	self
		assert: ((root := document root) isMemberOf: self userElementClass);
		assert: ((root elementAt: 'username') isMemberOf: self usernameElementClass);
		assert: ((root elementAt: 'real-name') isMemberOf: self realNameElementClass);
		assert: ((root elementAt: 'date-registered') isMemberOf: self dateElementClass);
		assert:
			((root elements copyAfter: (root elementAt: 'date-registered'))
				allSatisfy: [:each | each isMemberOf: self defaultElementClass]).! !

!XMLParserElementFactoryTest methodsFor: 'tests' stamp: 'JAAyer 11/21/2010 05:00'!
testElementsNameAndNamespaceMatching
	| factory document root |

	factory := XMLPluggableElementFactory new.
	factory
		handleElements: #('date-registered' 'date-last-active')
		namespaceURI: 'urn:user'
		withClass: self dateElementClass.
	factory
		handleElements: #('date-registered' 'date-last-active')
		withClass: self wrongElementClass.
	factory
		handleElements: #('table-name' 'not-table-name')
		withClass: self tableElementClass.

	document :=
		(XMLDOMParser on: self sampleUserXML)
			nodeFactory: factory;
			parseDocument.
	root := document root.
	self
		assert: ((root elementAt: 'date-registered') isMemberOf: self dateElementClass);
		assert: ((root elementAt: 'date-last-active') isMemberOf: self dateElementClass);
		assert: ((root elementAt: 'table-name') isMemberOf: self tableElementClass).! !

!XMLParserElementFactoryTest methodsFor: 'dummy element classes' stamp: 'JAAyer 11/20/2010 15:45'!
userElementClass
	^ Smalltalk at: #XMLTESTUserElement! !

!XMLParserElementFactoryTest methodsFor: 'dummy element classes' stamp: 'JAAyer 11/20/2010 15:45'!
usernameElementClass
	^ Smalltalk at: #XMLTESTUsernameElement! !

!XMLParserElementFactoryTest methodsFor: 'dummy element classes' stamp: 'JAAyer 11/20/2010 15:45'!
wrongElementClass
	^ Smalltalk at: #XMLTESTWrongElement! !

TestCase subclass: #XMLParserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser'!

!XMLParserTest methodsFor: 'source' stamp: 'AlexandreBergel 1/29/2009 11:49'!
addressBookXML
	^'<addressbook>
  <person employee-number="A0000" family-name="Gates" first-name="Bob">
    <contact-info><!!--Confidential--></contact-info>
    <address city="Los Angeles" number="1239" state="CA" street="Pine Rd."/>
    <job-info employee-type="Full-Time" is-manager="no" job-description="Manager"/>
    <manager employee-number="A0000"/>
  </person>
  <person employee-number="A7000" family-name="Brown"
    first-name="Robert" middle-initial="L.">
    <contact-info>
      <email address="robb@iro.ibm.com"/>
      <home-phone number="03-3987873"/>
    </contact-info>
    <address city="New York" number="344" state="NY" street="118 St."/>
    <job-info employee-type="Full-Time" is-manager="yes" job-description="Group Leader"/>
    <manager employee-number="A0000"/>
  </person>
  <person employee-number="A7890" family-name="DePaiva"
    first-name="Kassie" middle-initial="W.">
    <contact-info><!!-- Kassie''s agent phone: 03-987654 --></contact-info>
    <address city="Los Angeles" number="1234" state="CA" street="Pine Rd."/>
    <job-info employee-type="Full-Time" is-manager="no" job-description="Actor"/>
    <manager employee-number="A0000"/>
    <misc-info>One of the most talented actresses on Daytime. Kassie
      plays the devious and beautiful Blair Cramer on ABC&apos;s
      &quot;One Life To Live.&quot;</misc-info>
  </person>
  <person employee-number="A7987" family-name="Smith" first-name="Joe">
    <contact-info>
      <email address="joes@iro.ibm.com"/>
      <mobile-phone number="888-7657765"/>
      <home-phone number="03-8767898"/>
      <home-phone number="03-8767871"/>
    </contact-info>
    <address city="New York" number="12789" state="NY" street="W. 15th Ave."/>
    <job-info employee-type="Part-Time" is-manager="no" job-description="Hacker"/>
    <manager employee-number="A7000"/>
  </person>
</addressbook>
'! !

!XMLParserTest methodsFor: 'source' stamp: 'AlexandreBergel 1/29/2009 11:49'!
addressBookXMLWithDTD
	^'<?xml version="1.0" encoding="UTF-8"?>
<!!DOCTYPE addressbook SYSTEM "addressbook.dtd">
<?xml-stylesheet type="text/xsl" href="demo.xsl"?>
<addressbook>
  <person employee-number="A0000" family-name="Gates" first-name="Bob">
    <contact-info><!!--Confidential--></contact-info>
    <address city="Los Angeles" number="1239" state="CA" street="Pine Rd."/>
    <job-info employee-type="Full-Time" is-manager="no" job-description="Manager"/>
    <manager employee-number="A0000"/>
  </person>
  <person employee-number="A7000" family-name="Brown"
    first-name="Robert" middle-initial="L.">
    <contact-info>
      <email address="robb@iro.ibm.com"/>
      <home-phone number="03-3987873"/>
    </contact-info>
    <address city="New York" number="344" state="NY" street="118 St."/>
    <job-info employee-type="Full-Time" is-manager="yes" job-description="Group Leader"/>
    <manager employee-number="A0000"/>
  </person>
  <person employee-number="A7890" family-name="DePaiva"
    first-name="Kassie" middle-initial="W.">
    <contact-info><!!-- Kassie''s agent phone: 03-987654 --></contact-info>
    <address city="Los Angeles" number="1234" state="CA" street="Pine Rd."/>
    <job-info employee-type="Full-Time" is-manager="no" job-description="Actor"/>
    <manager employee-number="A0000"/>
    <misc-info>One of the most talented actresses on Daytime. Kassie
      plays the devious and beautiful Blair Cramer on ABC&apos;s
      &quot;One Life To Live.&quot;</misc-info>
  </person>
  <person employee-number="A7987" family-name="Smith" first-name="Joe">
    <contact-info>
      <email address="joes@iro.ibm.com"/>
      <mobile-phone number="888-7657765"/>
      <home-phone number="03-8767898"/>
      <home-phone number="03-8767871"/>
    </contact-info>
    <address city="New York" number="12789" state="NY" street="W. 15th Ave."/>
    <job-info employee-type="Part-Time" is-manager="no" job-description="Hacker"/>
    <manager employee-number="A7000"/>
  </person>
</addressbook>
'! !

!XMLParserTest methodsFor: 'benchmark' stamp: 'JAAyer 10/20/2010 11:39'!
benchmark1
	"
	XMLParserTest new benchmark1
	600 timesRepeat => 2131 (Alexandre Bergel 3/29/2010 09:09)
	"
	Smalltalk garbageCollect.
	^ [1500 timesRepeat: [SAXHandler parse: self addressBookXML]] timeToRun! !

!XMLParserTest methodsFor: 'benchmark' stamp: 'JAAyer 10/18/2010 08:15'!
benchmark2
	"
	XMLParserTest new benchmark2
	"
	Smalltalk garbageCollect.
	^ [1500 timesRepeat: [XMLDOMParser parse: self addressBookXML]] timeToRun! !

!XMLParserTest methodsFor: 'benchmark' stamp: 'JAAyer 10/18/2010 08:15'!
benchmark3
	"
	XMLParserTest new benchmark3
	"
	Smalltalk garbageCollect.
	^ [1500 timesRepeat: [self testParsing]] timeToRun! !

!XMLParserTest methodsFor: 'benchmark' stamp: 'JAAyer 11/22/2010 19:03'!
benchmark4
	| doc |
	"
	XMLParserTest new benchmark4
	"
	doc := XMLDOMParser
		parseDocumentFrom: self addressBookXML readStream
		useNamespaces: false.
	^ [500000 timesRepeat: [
		doc root firstElement
			elementAt: 'contact-info';
			elementAt: 'address';
			elementAt: 'job-info';
			elementAt: 'manager']] timeToRun.! !

!XMLParserTest methodsFor: 'source' stamp: 'jvds 8/9/2010 11:03'!
exampleCDATASectionXML
	^'<?xml version="1.0" encoding="UTF-8"?>
<test-data><!![CDATA[this is CDATA test content 1 < 2 & 5 > 3]]></test-data>'! !

!XMLParserTest methodsFor: 'source' stamp: 'DamienPollet 5/16/2010 19:25'!
exampleEncodedXML
	^'<?xml version="1.0" encoding="UTF-8"?>
<test-data>&#8230;</test-data>
'! !

!XMLParserTest methodsFor: 'tests' stamp: 'JAAyer 11/23/2010 15:46'!
testCDATASection
	| xmlDocument element output |

	xmlDocument :=
		(XMLDOMParser on: self exampleCDATASectionXML readStream)
			preservesCDataNodes: true;
			parseDocument.
	element := xmlDocument firstTagNamed: 'test-data'.

	self assert: element contentString = 'this is CDATA test content 1 < 2 & 5 > 3'.
	
	output := element printString.
	self assert: '<test-data><!![CDATA[this is CDATA test content 1 < 2 & 5 > 3]]></test-data>' = output
	
! !

!XMLParserTest methodsFor: 'tests-utf8' stamp: 'JAAyer 9/20/2010 17:40'!
testDecodingCharacters
	| xmlDocument element |

	xmlDocument := XMLDOMParser parse: self exampleEncodedXML.
	element := xmlDocument firstTagNamed: 'test-data'.

	self assert: element contentString first charCode = 8230.	

! !

!XMLParserTest methodsFor: 'tests' stamp: 'JAAyer 3/11/2010 09:06'!
testExampleAddressBook
	| tokenizer |
	"self debug: #testExampleAddressBook"

	tokenizer := XMLTokenizer on: self addressBookXML readStream.

	"We enumerate the first characters of the addressbook example.
	The file being parsed begins with <addressbook"
	self assert: tokenizer streamReader next = $<.
	self assert: tokenizer streamReader next = $a.	
	self assert: tokenizer streamReader next = $d.
	self assert: tokenizer streamReader next = $d.
	self assert: tokenizer streamReader next = $r.

	self
		shouldnt: [tokenizer streamReader next notNil] whileTrue
		raise: Error. ! !

!XMLParserTest methodsFor: 'tests' stamp: 'JAAyer 3/11/2010 09:04'!
testExampleAddressBookWithDTD
	| tokenizer |
	"XMLTokenizer exampleAddressBookWithDTD"

	tokenizer := XMLTokenizer on: self addressBookXMLWithDTD readStream.
	self
		shouldnt: [tokenizer streamReader next notNil] whileTrue
		raise: Error! !

!XMLParserTest methodsFor: 'tests' stamp: 'JAAyer 11/20/2010 14:32'!
testLineEndingsDoNotMatter
	| source docWithCrs docWithCrlfs |

	(source := String new writeStream)
		nextPutAll: '<foo>';
		cr;
		nextPutAll: 'bar';
		cr;
		nextPutAll: 'baz';
		cr;
		nextPutAll: '</foo>'.

	docWithCrs := XMLDOMParser parse: source contents.
	docWithCrlfs :=
		XMLDOMParser parse:
			(source contents
				copyReplaceAll: String cr
				with: String crlf).
	self assert: (docWithCrs root contentString) = (docWithCrlfs root contentString)! !

!XMLParserTest methodsFor: 'tests' stamp: 'JAAyer 8/11/2010 21:19'!
testParsing
	| xmlDocument root firstPerson numberOfPersons |
	"self debug: #testParsing"

	xmlDocument := XMLDOMParser parse: self addressBookXML.
	self assert: (xmlDocument isKindOf: XMLDocument).
	root := xmlDocument root.
	self assert: (root class == XMLElement).

	self assert: (root firstTagNamed: 'person') notNil.
	self assert: (root firstTagNamed: 'addressbook') notNil.

	self assert: (root firstTagNamed: 'addressbook') == root.

	numberOfPersons := 0.
	root tagsNamed: 'person' do: [:p | numberOfPersons := numberOfPersons + 1].
	self assert: numberOfPersons = 4.

	firstPerson := root firstTagNamed: 'person'.
	self assert: (firstPerson attributeAt: 'employee-number') = 'A0000'.
	self assert: (firstPerson attributeAt: 'family-name') = 'Gates'.
	self assert: (firstPerson attributeAt: 'first-name') = 'Bob'.
	self assert: firstPerson attributeNames asArray =
		#('employee-number' 'family-name' 'first-name').
	self assert: firstPerson attributeAssociations asArray =
		{('employee-number'->'A0000'). ('family-name'->'Gates'). ('first-name'->'Bob').}.

	self assert: (firstPerson @ 'employee-number') = (firstPerson attributeAt: 'employee-number').
	self assert: (firstPerson @ 'employee-number') = (firstPerson @ 'employee-number').! !

!XMLParserTest methodsFor: 'tests' stamp: 'JAAyer 3/14/2010 12:54'!
testParsingCharacters
	| parser |
	"This test is actually not that useful. This is not the proper way of using the parser. This test is here just for specification purpose"
	"self debug: #testParsingCharacters"

	parser := SAXDriver on: self addressBookXML readStream.

	self assert: parser streamReader next = $<.
	self assert: parser streamReader next = $a.
	self assert: parser streamReader next = $d.
	self assert: parser streamReader next = $d.
	self assert: parser streamReader next = $r.! !

!XMLParserTest methodsFor: 'tests-xmltokenizer' stamp: 'JAAyer 3/11/2010 09:06'!
testTokenizerAddressBook
	| tokenizer |
	tokenizer := XMLTokenizer on: self addressBookXML readStream.
	[tokenizer streamReader next notNil] whileTrue! !

!XMLParserTest methodsFor: 'tests-xmltokenizer' stamp: 'JAAyer 3/11/2010 09:06'!
testTokenizerAddressBookWithDTD
	| tokenizer |
	tokenizer := XMLTokenizer on: self addressBookXMLWithDTD readStream.
	[tokenizer streamReader next notNil] whileTrue! !

XMLNode subclass: #XMLMockNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Nodes'!

!XMLMockNode methodsFor: 'printing' stamp: 'JAAyer 11/15/2010 05:02'!
printXMLOn: aWriter
	aWriter string: self class name asString, '(', self identityHash printString, ') '! !

XMLMockNode subclass: #XMLMockObserverNode
	instanceVariableNames: 'lastNotificationArguments'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Nodes'!

!XMLMockObserverNode methodsFor: 'accessing' stamp: 'JAAyer 10/28/2010 14:52'!
lastNotificationArguments
	^ lastNotificationArguments! !

!XMLMockObserverNode methodsFor: 'accessing' stamp: 'JAAyer 10/28/2010 14:52'!
lastNotificationArguments: anArray
	lastNotificationArguments := anArray! !

!XMLMockObserverNode methodsFor: 'notifications' stamp: 'JAAyer 10/28/2010 14:53'!
mockNotifyWith: aFirstObject
	lastNotificationArguments := Array with: aFirstObject! !

!XMLMockObserverNode methodsFor: 'notifications' stamp: 'JAAyer 10/28/2010 14:53'!
mockNotifyWith: aFirstObject with: aSecondObject
	lastNotificationArguments := Array with: aFirstObject with: aSecondObject! !

!XMLMockObserverNode methodsFor: 'notifications' stamp: 'JAAyer 10/28/2010 14:54'!
mockNotifyWith: aFirstObject with: aSecondObject with: aThirdArgument
	lastNotificationArguments := Array with: aFirstObject with: aSecondObject with: aThirdArgument! !

XMLNodeWithChildren subclass: #XMLMockNodeWithChildren
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Nodes'!

!XMLMockNodeWithChildren methodsFor: 'private' stamp: 'JAAyer 10/28/2010 21:33'!
nodesClass
	^ XMLNodeList! !

OrderedCollectionTest subclass: #XMLOrderedListTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Nodes'!

!XMLOrderedListTest methodsFor: 'coverage' stamp: 'JAAyer 8/4/2010 20:10'!
classToBeTested
	^ XMLOrderedList! !

!XMLOrderedListTest methodsFor: 'test - creation' stamp: 'JAAyer 8/4/2010 20:09'!
collectionClass
	^ XMLOrderedList! !

XMLNodeWithElements subclass: #XMLMockNodeWithElements
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Tests-Parser-Nodes'!

!XMLMockNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 7/12/2010 03:21'!
document
	^ nil! !

!XMLMockNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/24/2010 13:36'!
document: aDocument
! !
